package cn.iocoder.yudao.module.reward.controller.admin.redeem;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.module.reward.api.reward.RedeemRewardApi;
import cn.iocoder.yudao.module.reward.api.reward.dto.RedeemRewardReqVO;
import cn.iocoder.yudao.module.reward.api.reward.dto.RedeemRewardRespDTO;
import cn.iocoder.yudao.module.reward.service.redeem.RewardRedeemService;
import cn.iocoder.yudao.module.reward.vo.reward.*;
import cn.iocoder.yudao.module.reward.vo.reward.RewardRedeemPageReqVO;
import cn.iocoder.yudao.module.reward.vo.reward.RewardRedeemPageRespVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Validated
@RestController
@Api(tags = "管理后台 - 兑换记录管理")
@RequestMapping("/reward/redeem")
public class RedeemRewardController {

    @Resource
    private RedeemRewardApi redeemRewardApi;

    @Resource
    private RewardRedeemService rewardRedeemService;

    @PostMapping("record/page")
    @ApiOperation("列表")
    public CommonResult<PageResult<RewardRedeemPageRespVO>> pageInfo(@Valid @RequestBody RewardRedeemPageReqVO reqVO) {
        return success(rewardRedeemService.pageInfo(reqVO));
    }

    @PostMapping("record/detail")
    @ApiOperation("详情")
    public CommonResult<RewardRedeemPageRespVO> detail(@Valid @RequestBody RewardRedeemBaseVO request) {
        return success(rewardRedeemService.detail(request.getId()));
    }

    @PostMapping("record/update")
    @ApiOperation("更新")
    public CommonResult<Boolean> update(@Valid @RequestBody RewardRedeemUpdateReqVO request) {
        return success(rewardRedeemService.updateRecord(request));
    }

    @PostMapping("record/verify")
    @ApiOperation("核销")
    public CommonResult<Boolean> verify(@Valid @RequestBody RewardRedeemVerifyReqVO request) {
        return success(rewardRedeemService.verify(request));
    }

    @PostMapping("record/batch/verify")
    @ApiOperation("批量核销")
    public CommonResult<Boolean> batchVerify(@Valid @RequestBody RewardRedeemBatchVerifyReqVO request) {
        return success(rewardRedeemService.batchVerify(request));
    }

    @PostMapping("record/verify-back")
    @ApiOperation("核销回退")
    public CommonResult<Boolean> verifyBack(@Valid @RequestBody RewardRedeemVerifyBackReqVO redeemIds) {
        return success(rewardRedeemService.verifyBack(redeemIds.getRedeemIds()));
    }

    @PostMapping("record/export")
    @ApiOperation("导出")
    public CommonResult<Boolean> export(@Valid @RequestBody RewardRedeemPageReqVO reqVO) {
        return success(rewardRedeemService.export(reqVO));
    }

    @PostMapping("record/cancel/check")
    @ApiOperation("撤销检查")
    public CommonResult<RedeemCancelCheckRespVO> cancelCheck(@Valid @RequestBody RewardRedeemIdReqVO req) {
        return success(rewardRedeemService.cancelCheck(req));
    }

    @PostMapping("record/cancel")
    @ApiOperation("撤销")
    public CommonResult<Boolean> cancel(@Valid @RequestBody RewardRedeemIdReqVO req) {
        return success(rewardRedeemService.cancel(req));
    }

    @PostMapping("record/import/template")
    @ApiOperation("导入模板下载")
    public void importTemplate(HttpServletResponse response) throws IOException {
        // 手动创建导出 demo
        List<RedeemInfoImportExcelVO> list = Arrays.asList(
                RedeemInfoImportExcelVO.builder().id(String.valueOf(11111)).redeemType("上门领取").courierCompany("顺丰空运").expenses(BigDecimal.valueOf(11.11))
                        .currency("人民币").expressDate("2024-01-01 11:22:33").expressNo("SDSD212212").remark("备注备注").expressSender("系统管理员")
                        .build(),
                RedeemInfoImportExcelVO.builder().id(String.valueOf(11111)).redeemType("包邮到家").courierCompany("顺丰空运").expenses(BigDecimal.valueOf(11.11))
                        .currency("美元").expressDate("2024-01-01 11:22:33").expressNo("SDSD212212").remark("备注备注").expressSender("系统管理员")
                        .build());
        // 输出
        ExcelUtils.write(response, "兑换信息导入模板.xls", "兑换信息", RedeemInfoImportExcelVO.class, list);
    }

    @PostMapping(value = "record/import", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    @ApiOperation("兑换信息导入")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "file", value = "Excel 文件", required = true, dataTypeClass = MultipartFile.class)
    })
    public CommonResult<RecordInfoImportRespVO> recordImport(@RequestParam("file") MultipartFile file) throws IOException {
        List<RedeemInfoImportExcelVO> list = ExcelUtils.read(file, RedeemInfoImportExcelVO.class);
        return success(rewardRedeemService.recordImport(list));
    }


    @PostMapping("/single")
    @ApiOperation("兑换礼品")
    @Idempotent(timeout = 5)
    @PreAuthorize("@ss.hasPermission('ecw:giftManagement:exchange')")
    public CommonResult<Boolean> redeemReward(@Valid @RequestBody RedeemRewardReqVO redeemRewardReqVO) {
        return success(redeemRewardApi.redeemReward(redeemRewardReqVO));
    }

    @PostMapping("/batch")
    @ApiOperation("批量兑换礼品")
    @Idempotent(timeout = 5)
    @PreAuthorize("@ss.hasPermission('ecw:giftManagement:exchange')")
    public CommonResult<List<RedeemRewardRespDTO>> redeemRewards(@Valid @RequestBody List<RedeemRewardReqVO> redeemRewardReqVOList) {
        return success(redeemRewardApi.redeemRewards(redeemRewardReqVOList));
    }
}
