package cn.iocoder.yudao.module.reward.listener;

import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.apollo.core.event.export.RewardRedeemRecordExportEvent;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.module.infra.api.file.FileApi;
import cn.iocoder.yudao.module.infra.api.file.dto.FileRespDto;
import cn.iocoder.yudao.module.infra.dal.dataobject.file.FileDO;
import cn.iocoder.yudao.module.reward.service.redeem.RewardRedeemService;
import cn.iocoder.yudao.module.reward.vo.reward.RewardRedeemPageReqVO;
import cn.iocoder.yudao.module.reward.vo.reward.RewardRedeemPageRespVO;
import cn.iocoder.yudao.module.reward.vo.reward.RewardRedeemRecordExportVO;
import cn.iocoder.yudao.module.system.framework.ue.UeProperties;
import com.alibaba.fastjson.JSONObject;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileInputStream;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.excel.constant.ExportConstant.DATA_FORMAT;

@Component
@AllArgsConstructor
@Slf4j
public class RewardRedeemRecordExportListener {

    private RewardRedeemService rewardRedeemService;
    private UeProperties ueProperties;
    private FileApi fileService;

    @EventListener(RewardRedeemRecordExportEvent.class)
    public void listen(RewardRedeemRecordExportEvent event) {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern(DATA_FORMAT);
        String nowTime = formatter.format(LocalDateTime.now());
        String dir = ueProperties.getTempDir().concat("/rewardRedeemRecord/excel/");
        File fileDir = new File(dir);
        if (!fileDir.exists()){
            // 不存在则创建一个目录
            fileDir.mkdirs();
        }
        String fileName = event.getUserId().toString().concat(StrUtil.DASHED).concat(event.getUserType().toString()).concat(StrUtil.DASHED).concat(nowTime).concat("shipment_preload.xlsx");
        RewardRedeemPageReqVO request = JSONObject.parseObject(event.getRequestParams(), RewardRedeemPageReqVO.class);
        Integer totalCount = rewardRedeemService.exportCount(request);
        int size = 500;
        int totalPage = totalCount%size == 0 ? totalCount/size: totalCount/size + 1;
        List<RewardRedeemRecordExportVO> result = new ArrayList<>();
        for (int i = 1; i <= totalPage ; i++) {
            request.setPageNo(i);
            request.setPageSize(size);
            PageResult<RewardRedeemPageRespVO> rewardRedeemPageRespVOPageResult = rewardRedeemService.pageInfo(request);
            result.addAll(convert2ExcelVo(rewardRedeemPageRespVOPageResult.getList()));
        }
        try {
            String path = ExcelUtils.write(dir, fileName, "数据", RewardRedeemRecordExportVO.class, result);
            // 获取到临时文件
            File file = new File(path);
            // 创建FileInputStream对象
            FileInputStream fileInputStream = new FileInputStream(file);
            // 读取文件内容
            byte[] fileBytes = new byte[(int) file.length()];
            fileInputStream.read(fileBytes);
            // 关闭文件流
            fileInputStream.close();
            // 将文件上传到资源服务器
            FileRespDto fileRespDto = fileService.createFile(dir, fileName, fileBytes);
            event.setPath(fileRespDto.getPath());
            event.setFileName(fileRespDto.getFileName());
            event.setUrl(fileRespDto.getUrl());
            event.setFileId(fileRespDto.getId());
        } catch (Exception e) {
            log.info("reward redeem record export listener exception", e);
            event.setResult(e.getMessage());
        }
    }

    private Collection<RewardRedeemRecordExportVO> convert2ExcelVo(List<RewardRedeemPageRespVO> list) {
        return list.stream().map(item -> {
            RewardRedeemRecordExportVO exportVO = new RewardRedeemRecordExportVO();
            exportVO.setId(item.getRedemptionNumber());
            exportVO.setStatus(item.getStatus());
            exportVO.setRewardCode(item.getRewardCode());
            exportVO.setRewardTitle(item.getRewardTitleZh());
            exportVO.setMemberName(item.getMemberNameZh());
            exportVO.setRewardCount(item.getRewardCount());
            exportVO.setRedeemType(item.getRedeemType());
            exportVO.setEntrance(item.getEntrance());
            exportVO.setExpenses(item.getExpenses());
            exportVO.setCurrencyTitle(item.getCurrencyTitleZh());
            exportVO.setRecipientName(item.getRecipientName());
            exportVO.setRecipientPhoneNum(item.getRecipientPhoneNum());
            exportVO.setRecipientAddress(item.getRecipientAddress());
            exportVO.setRedeemer(item.getRedeemer());
            exportVO.setRedemptionTime(item.getRedemptionTime());
            exportVO.setCourierCompanyName(item.getCourierCompanyName());
            exportVO.setExpressNo(item.getExpressNo());
            exportVO.setExpressDate(item.getExpressDate());
            exportVO.setExpressSender(item.getExpressSender());
            exportVO.setRemark(item.getRemark());
            exportVO.setVerifyUser(item.getVerifyUser());
            exportVO.setVerifyTime(item.getVerifyTime());
            exportVO.setCreatorName(item.getCreatorName());
            exportVO.setCreateTime(item.getCreateTime());
            exportVO.setUpdaterName(item.getUpdaterName());
            exportVO.setUpdateTime(item.getUpdateTime());
            return exportVO;
        }).collect(Collectors.toList());
    }
}
