package cn.iocoder.yudao.module.reward.service.reward;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.reward.controller.app.reward.vo.AppRewardBackVO;
import cn.iocoder.yudao.module.reward.controller.app.reward.vo.AppRewardListBackVO;
import cn.iocoder.yudao.module.reward.controller.app.reward.vo.AppRewardListReqVO;
import cn.iocoder.yudao.module.reward.dal.dataobject.reward.RewardDO;
import cn.iocoder.yudao.module.reward.vo.reward.*;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 礼品 Service 接口
 *
 * @author 系统管理员
 */
public interface RewardService extends IService<RewardDO> {

    /**
     * 创建礼品
     * @param createReqVO 创建信息
     * @return 编号
     */
    List<Long> create(@Valid RewardCreateReqVO createReqVO);

    /**
     * 更新礼品
     * @param updateReqVO 更新信息
     */
    void update(@Valid RewardUpdateReqVO updateReqVO);

    /**
     * 删除礼品
     * @param id 编号
     */
    void delete(Long id);

    /**
     * 获得礼品
     * @param id 编号
     * @return 礼品
     */
    RewardDO get(Long id);

    /**
     * 获得礼品列表
     * @param ids 编号
     * @return 礼品列表
     */
    List<RewardDO> getList(Collection<Long> ids);

    /**
     * 获得礼品分页
     * @param page 分页查询
     * @param query 查询
     * @return 礼品分页
     */
    PageResult<RewardDO> getPage(RewardQueryVO query, PageVO page);

    /**
     * 获得礼品列表, 用于 Excel 导出
     * @param query 查询
     * @return 礼品列表
     */
    List<RewardDO> getList(RewardQueryVO query);

    /**
     * 复制礼品
     * @param id 编号
     * @return 新的礼品id
     */
    Long copyReward(Long id);

    void delayReward(RewardDelayReqVO delayVO);

    void updateStatus(RewardStatusReqVO statusVO);

    Boolean validateExpire(RewardDO rewardDO);
    /**
     * 客户端获得礼品列表
     * @param reqVO
     * @return
     */
    AppRewardListBackVO getListByMemberInfo(AppRewardListReqVO reqVO);

    AppRewardBackVO getRewardDetailForUser(Long id);
}
