package cn.iocoder.yudao.module.reward.service.reward;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.dict.core.util.DictFrameworkUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseDO;
import cn.iocoder.yudao.module.depository.dal.mysql.warehouse.WarehouseMapper;
import cn.iocoder.yudao.module.ecw.dal.dataobject.region.RegionDO;
import cn.iocoder.yudao.module.ecw.service.region.RegionService;
import cn.iocoder.yudao.module.member.api.user.MemberUserApi;
import cn.iocoder.yudao.module.member.api.user.dto.UserRespDTO;
import cn.iocoder.yudao.module.reward.controller.app.reward.vo.AppRewardBackVO;
import cn.iocoder.yudao.module.reward.controller.app.reward.vo.AppRewardListBackVO;
import cn.iocoder.yudao.module.reward.controller.app.reward.vo.AppRewardListReqVO;
import cn.iocoder.yudao.module.reward.convert.reward.RewardConvert;
import cn.iocoder.yudao.module.reward.dal.dataobject.reward.RewardDO;
import cn.iocoder.yudao.module.reward.dal.mysql.reward.RewardMapper;
import cn.iocoder.yudao.module.reward.enums.QuantitativeRelationSymbolEnum;
import cn.iocoder.yudao.module.reward.enums.RewardPickMethedEnum;
import cn.iocoder.yudao.module.reward.enums.RewardStatusEnum;
import cn.iocoder.yudao.module.reward.util.RewardGenCodeUtils;
import cn.iocoder.yudao.module.reward.vo.reward.*;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.time.Instant;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.reward.enums.ErrorCodeConstants.*;

/**
 * 礼品 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class RewardServiceImpl extends AbstractService<RewardMapper, RewardDO> implements RewardService {

    @Resource
    private RewardMapper rewardMapper;
    @Resource
    private WarehouseMapper warehouseMapper;
    @Resource
    private RegionService regionService;
    @Resource
    private MemberUserApi memberUserApi;
    @Resource
    private RewardGenCodeUtils rewardGenCodeUtils;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<Long> create(RewardCreateReqVO createReqVO) {
        verifyCreateVO(createReqVO);
        // 插入
        RewardDO rewardDO = RewardConvert.INSTANCE.convert(createReqVO);
        ArrayList<Long> ids = new ArrayList<>();
        for (RewardCreateReqVO.NodeAndPoints wareHouses : createReqVO.getNodeIds()) {
            rewardDO.setPointsRequire(wareHouses.getPoints());
            // 剩余数量若没传给默认值0
            if (wareHouses.getQuantityRemain() == null) {
                rewardDO.setQuantityRemain(0);
            } else if (wareHouses.getQuantityRemain() > 0) {
                rewardDO.setQuantityRemain(wareHouses.getQuantityRemain());
            } else {
                rewardDO.setQuantityRemain(0);
            }
            for (Integer wareHouse : wareHouses.getWareHouses()) {
                if (wareHouse == null || wareHouse < 0) {
                    throw exception(REWARD_NODE_AND_POINTS_LIST_ERROR);
                }
                rewardDO.setNodeId(wareHouse);
                rewardDO.setCode(rewardGenCodeUtils.generateRewardCode());
                rewardMapper.insert(rewardDO);
                ids.add(rewardDO.getId());
                rewardDO.setId(null);
            }
        }
        // 返回
        return ids;
    }

    private void verifyCreateVO(RewardCreateReqVO createReqVO) {
        // 校验领取方式
        List<String> pickMethodList = Arrays.asList(createReqVO.getPickMethod().split(","));
        if (!pickMethodList.contains(String.valueOf(RewardPickMethedEnum.SELF_PICKUP.getValue())) && !pickMethodList.contains(String.valueOf(RewardPickMethedEnum.SEND_HOME_FREE.getValue())) && !pickMethodList.contains(String.valueOf(RewardPickMethedEnum.SEND_HOME_PAY.getValue()))) {
            throw exception(REWARD_PICK_METHOD_NOT_ALLOW);
        }
        // 活动开始时间不能比活动结束时间晚，活动结束时间不能比当前时间早
        if (ObjectUtils.allNotNull(createReqVO.getStartTime(), createReqVO.getEndTime())) {
            if (createReqVO.getStartTime().after((createReqVO.getEndTime())) || createReqVO.getEndTime().before(Date.from(Instant.now()))) {
                throw exception(REWARD_START_OR_END_TIME_NOT_ALLOW);
            }
        } else if (ObjectUtils.allNull(createReqVO.getStartTime(), createReqVO.getEndTime())) {
        } else {
            throw exception(REWARD_START_OR_END_TIME_NOT_ALLOW);
        }
        // 创建的礼品状态只能是启用或未启用
        if (createReqVO.getStatus() != RewardStatusEnum.ENABLED.getValue() && createReqVO.getStatus() != RewardStatusEnum.DISABLED.getValue()) {
            throw exception(REWARD_STATUS_NOT_ALLOW_CREATE);
        }
        // 校验节点和所需积分
        List<RewardCreateReqVO.NodeAndPoints> nodeIds = createReqVO.getNodeIds();
        if (CollectionUtil.isEmpty(nodeIds)) {
            throw exception(REWARD_NODE_AND_POINTS_LIST_ERROR);
        } else {
            for (RewardCreateReqVO.NodeAndPoints nodeId : nodeIds) {
                if (nodeId.getPoints() == null || nodeId.getWareHouses() == null) {
                    throw exception(REWARD_NODE_AND_POINTS_LIST_ERROR);
                }
                if (nodeId.getPoints() <= 0) {
                    throw exception(REWARD_NODE_AND_POINTS_LIST_ERROR);
                }
            }
        }
    }

    @Override
    public void update(RewardUpdateReqVO updateReqVO) {
        // 校验存在
        RewardDO rewardDO = rewardMapper.selectById(updateReqVO.getId());
        if (rewardDO == null) {
            throw exception(REWARD_NOT_EXISTS);
        }
        RewardDO updateObj = new RewardDO();

        // 此处为了兼容前端传值
        List<RewardCreateReqVO.NodeAndPoints> nodeIds = updateReqVO.getNodeIds();
        if (CollectionUtil.isEmpty(nodeIds)) {
            throw exception(REWARD_NODE_AND_POINTS_LIST_ERROR);
        }
        RewardCreateReqVO.NodeAndPoints nodeAndPoints = nodeIds.get(0);
        //未启用可编辑所有内容
        if (rewardDO.getStatus() == RewardStatusEnum.DISABLED.getValue()) {
            // 校验领取方式
            List<String> pickMethodList = Arrays.asList(updateReqVO.getPickMethod().split(","));
            if (StringUtils.isNotBlank(updateReqVO.getPickMethod()) && !pickMethodList.contains(String.valueOf(RewardPickMethedEnum.SELF_PICKUP.getValue())) && !pickMethodList.contains(String.valueOf(RewardPickMethedEnum.SEND_HOME_FREE.getValue())) && !pickMethodList.contains(String.valueOf(RewardPickMethedEnum.SEND_HOME_PAY.getValue()))) {
                throw exception(REWARD_PICK_METHOD_NOT_ALLOW);
            }
            if (ObjectUtils.allNotNull(updateReqVO.getStartTime(), updateReqVO.getEndTime())) {
                if (updateReqVO.getStartTime().after((updateReqVO.getEndTime())) || updateReqVO.getEndTime().before(Date.from(Instant.now()))) {
                    throw exception(REWARD_START_OR_END_TIME_NOT_ALLOW);
                }
            } else if (ObjectUtils.allNull(updateReqVO.getStartTime(), updateReqVO.getEndTime())) {
            } else {
                throw exception(REWARD_START_OR_END_TIME_NOT_ALLOW);
            }
            updateObj = RewardConvert.INSTANCE.convert(updateReqVO);
            updateObj.setPointsRequire(nodeAndPoints.getPoints());
            updateObj.setNodeId(nodeAndPoints.getWareHouses().get(0));
            updateObj.setQuantityRemain(nodeAndPoints.getQuantityRemain());

            //已启用可编辑剩余数量和备注
        } else if (rewardDO.getStatus() == RewardStatusEnum.ENABLED.getValue()) {
            updateObj.setId(updateReqVO.getId());
            updateObj.setQuantityRemain(nodeAndPoints.getQuantityRemain());
            updateObj.setRemarkEn(updateReqVO.getRemarkEn());
            updateObj.setRemarkFr(updateReqVO.getRemarkFr());
            updateObj.setRemarkZh(updateReqVO.getRemarkZh());
        } else {
            throw exception(REWARD_STATUS_NOT_ALLOW_UPDATE);
        }
        // 更新
        rewardMapper.updateById(updateObj);
    }

    @Override
    public void delete(Long id) {
        RewardDO rewardDO = rewardMapper.selectById(id);
        if (rewardDO == null) {
            throw exception(REWARD_NOT_EXISTS);
        }
        //只有未启用可以删除
        if (rewardDO.getStatus() != RewardStatusEnum.DISABLED.getValue()) {
            throw exception(REWARD_STATUS_NOT_ALLOW_DELETE);
        }
        // 删除
        rewardMapper.deleteById(id);
    }

    private void validateExists(Long id) {
        if (rewardMapper.selectById(id) == null) {
            throw exception(REWARD_NOT_EXISTS);
        }
    }

    @Override
    public RewardDO get(Long id) {
        RewardDO rewardDO = rewardMapper.selectById(id);
        //校验是否过期
        if (rewardDO != null) {
            validateExpire(rewardDO);
        }
        return rewardDO;
    }

    @Override
    public List<RewardDO> getList(Collection<Long> ids) {
        List<RewardDO> rewardDOS = rewardMapper.selectBatchIds(ids);
        //校验是否过期
        rewardDOS.forEach(this::validateExpire);
        return rewardDOS;
    }

    @Override
    public PageResult<RewardDO> getPage(RewardQueryVO query, PageVO page) {

        LambdaQuery<RewardDO> lambdaQuery = new LambdaQuery<>();
        //礼品ID
        lambdaQuery.eqIfPresent(RewardDO::getCode, query.getCode());
        //礼品名称，模糊匹配三种语言
        if (StringUtils.isNotBlank(query.getTitle())) {
            lambdaQuery.and(wrapper ->
                    wrapper.like(RewardDO::getTitleZh, query.getTitle())
                            .or()
                            .like(RewardDO::getTitleEn, query.getTitle())
                            .or()
                            .like(RewardDO::getTitleFr, query.getTitle())
            );
        }
        //兑换积分
        if (query.getPointsRequireSymbol() != null && query.getPointsRequire() != null) {
            lambdaQuery.gt(query.getPointsRequireSymbol() == QuantitativeRelationSymbolEnum.GT.getValue(), RewardDO::getPointsRequire, query.getPointsRequire())
                    .eq(query.getPointsRequireSymbol() == QuantitativeRelationSymbolEnum.EQ.getValue(), RewardDO::getPointsRequire, query.getPointsRequire())
                    .lt(query.getPointsRequireSymbol() == QuantitativeRelationSymbolEnum.LT.getValue(), RewardDO::getPointsRequire, query.getPointsRequire());
        }
        //提货点id，领取方式，状态
        lambdaQuery.eqIfPresent(RewardDO::getNodeId, query.getNodeId())
                .likeIfPresent(RewardDO::getPickMethod, query.getPickMethod())
                .eqIfPresent(RewardDO::getStatus, query.getStatus());
        //判断memberId，筛选会员所在城市网点礼品
        if (query.getMemberId() != null && query.getNodeId() == null) {
            List<Long> nodes = getMemberCityNodes(query.getMemberId());
            lambdaQuery.in(CollectionUtil.isNotEmpty(nodes), RewardDO::getNodeId, nodes);
        }
        //备注，模糊匹配三种语言
        if (StringUtils.isNotBlank(query.getRemark())) {
            lambdaQuery.and(wrapper ->
                    wrapper.like(RewardDO::getRemarkZh, query.getRemark())
                            .or()
                            .like(RewardDO::getRemarkEn, query.getRemark())
                            .or()
                            .like(RewardDO::getRemarkFr, query.getRemark())
            );
        }
        //剩余数量
        if (query.getQuantityRemainSymbol() != null && query.getQuantityRemain() != null) {
            lambdaQuery.gt(query.getQuantityRemainSymbol() == QuantitativeRelationSymbolEnum.GT.getValue(), RewardDO::getQuantityRemain, query.getQuantityRemain())
                    .eq(query.getQuantityRemainSymbol() == QuantitativeRelationSymbolEnum.EQ.getValue(), RewardDO::getQuantityRemain, query.getQuantityRemain())
                    .lt(query.getQuantityRemainSymbol() == QuantitativeRelationSymbolEnum.LT.getValue(), RewardDO::getQuantityRemain, query.getQuantityRemain());
        }
        //创建时间
        lambdaQuery.betweenIfPresent(RewardDO::getCreateTime, query.getBeginCreateTime(), query.getEndCreateTime())
                .orderByDesc(RewardDO::getId);
        PageResult<RewardDO> rewardDOPageResult = rewardMapper.selectPage(page, lambdaQuery);
        //校验是否过期
        rewardDOPageResult.getList().forEach(this::validateExpire);
        return rewardDOPageResult;
    }

    @Override
    public List<RewardDO> getList(RewardQueryVO query) {
        return rewardMapper.selectList(query);
    }

    @Override
    public Long copyReward(Long id) {
        RewardDO reward = rewardMapper.selectById(id);
        if (reward == null) {
            throw exception(REWARD_NOT_EXISTS);
        }
        //重新生成礼品ID
        reward.setCode(rewardGenCodeUtils.generateRewardCode());
        //设置为未启用
        reward.setStatus(RewardStatusEnum.DISABLED.getValue());
        //已兑换次数清零
        reward.setExchangeCount(0);

        reward.setId(null);
        reward.setCreateTime(null);
        reward.setUpdateTime(null);
        reward.setDeleted(null);
        rewardMapper.insert(reward);
        return reward.getId();
    }

    @Override
    public void delayReward(RewardDelayReqVO delayVO) {
        RewardDO reward = rewardMapper.selectById(delayVO.getId());
        if (reward == null) {
            throw exception(REWARD_NOT_EXISTS);
        }
        //只允许延期已启用状态礼品
        if (reward.getStatus() != RewardStatusEnum.ENABLED.getValue()) {
            throw exception(REWARD_STATUS_NOT_ALLOW_DELAY);
        }
        Instant now = Instant.now();
        //结束时间不能小于当前时间
        if (!delayVO.getEndTime().toInstant().isAfter(now)) {
            throw exception(REWARD_ENDTIME_ERROR);
        }
        rewardMapper.updateById(RewardConvert.INSTANCE.convertDelayReqVO(delayVO));
    }

    @Override
    public void updateStatus(RewardStatusReqVO statusVO) {
        this.validateExists(statusVO.getId());
        RewardDO rewardDO = rewardMapper.selectById(statusVO.getId());
        Integer oldStatus = rewardDO.getStatus();
        Integer newStatus = statusVO.getStatus();
        switch (newStatus) {
            //关闭：未启用情况下，不可进行关闭；关闭后状态为已关闭，不支持兑换
            case 3:
                if (oldStatus == RewardStatusEnum.DISABLED.getValue()) {
                    throw exception(REWARD_STATUS_NOT_ALLOW_ClOSE);
                } else {
                    rewardMapper.updateById(RewardConvert.INSTANCE.convertStatusReqVO(statusVO));
                }
                break;
            //启用：有效期有效且未启用可进行点击启用
            case 1:
                if (oldStatus == RewardStatusEnum.DISABLED.getValue()) {
                    if (rewardDO.getEndTime().toInstant().isBefore(Instant.now())) {
                        throw exception(REWARD_STATUS_NOT_ALLOW_ENABLE);
                    }
                    rewardMapper.updateById(RewardConvert.INSTANCE.convertStatusReqVO(statusVO));
                } else {
                    throw exception(REWARD_STATUS_NOT_ALLOW_ENABLE);
                }
                break;
            default:
                throw exception(REWARD_STATUS_CHANGE_ERROR);
        }
    }

    @Override
    public AppRewardListBackVO getListByMemberInfo(AppRewardListReqVO reqVO) {
        AppRewardListBackVO appRewardListBackVO = new AppRewardListBackVO();
        LambdaQuery<RewardDO> rewardDOLambdaQuery = new LambdaQuery<>();
        rewardDOLambdaQuery.eq(RewardDO::getStatus, RewardStatusEnum.ENABLED.getValue())
                .gt(RewardDO::getQuantityRemain, 0)
                .le(RewardDO::getStartTime, new Date())
                .ge(RewardDO::getEndTime, new Date());
        // 如果有网点，直接根据网点查
        if (reqVO.getNodeId() != null) {
            rewardDOLambdaQuery.eq(RewardDO::getNodeId, reqVO.getNodeId());
        } else {
            UserRespDTO user = memberUserApi.getUser(reqVO.getMemberId());
            Integer userCity = user.getCity();
            if (userCity == null) {
                userCity = 0;
            }
            Integer userCountry = user.getCountry();
            if (userCountry == null) {
                userCountry = 130;
            }
            List<WarehouseDO> warehouseList;
            // 根据用户城市获取
            warehouseList = warehouseMapper.selectList(new LambdaQueryWrapperX<WarehouseDO>().eq(WarehouseDO::getShi, userCity.longValue()));
            appRewardListBackVO.setCity(userCity.longValue());
            appRewardListBackVO.setCountry(userCountry.longValue());
            if (CollectionUtil.isEmpty(warehouseList)) {
                // 根据用户国家获取
                warehouseList = warehouseMapper.selectList(new LambdaQueryWrapperX<WarehouseDO>().eq(WarehouseDO::getGuojia, userCountry.longValue()));
                appRewardListBackVO.setCity(null);
            }
            // 国家和城市都获取不到
            if (CollectionUtil.isEmpty(warehouseList)) {
                warehouseList = warehouseMapper.selectList(new LambdaQueryWrapperX<WarehouseDO>().eq(WarehouseDO::getGuojia, 130L));
                appRewardListBackVO.setCountry(130L);
            }
            List<Long> nodeIds = warehouseList.stream().map(WarehouseDO::getId).collect(Collectors.toList());
            if (CollectionUtil.isNotEmpty(nodeIds)) {
                rewardDOLambdaQuery.in(RewardDO::getNodeId, nodeIds);
            }
        }
        List<RewardDO> rewardDOS = rewardMapper.selectList(rewardDOLambdaQuery);
        if (rewardDOS != null && !rewardDOS.isEmpty()) {
            List<AppRewardBackVO> collect = rewardDOS.stream().map(RewardConvert.INSTANCE::convertAppVO).collect(Collectors.toList());
            collect.forEach(this::setPickMethod);
            appRewardListBackVO.setList(collect);
            return appRewardListBackVO;
        }
        appRewardListBackVO.setList(Collections.emptyList());
        return appRewardListBackVO;
    }

    @Override
    public AppRewardBackVO getRewardDetailForUser(Long id) {
        RewardDO rewardDO = rewardMapper.selectById(id);
        AppRewardBackVO appRewardBackVO = new AppRewardBackVO();
        BeanUtils.copyProperties(rewardDO, appRewardBackVO);
        setPickMethod(appRewardBackVO);
        WarehouseDO warehouse = warehouseMapper.selectById(Long.valueOf(rewardDO.getNodeId()));
        if (warehouse != null) {
            RegionDO country = regionService.getRegion(warehouse.getGuojia());
            RegionDO city = regionService.getRegion(warehouse.getShi());
            appRewardBackVO.setNodeZh(country.getTitleZh() + "、" + city.getTitleZh() + "、" + warehouse.getTitleZh());
            appRewardBackVO.setNodeEn(country.getTitleEn() + "、" + city.getTitleEn() + "、" + warehouse.getTitleEn());
        }
        return appRewardBackVO;
    }

    private void setPickMethod(AppRewardBackVO appRewardBackVO) {
        String[] splitPickMethod = appRewardBackVO.getPickMethod().split(",");
        for (String pickMethod : splitPickMethod) {
            AppRewardBackVO.PickMethod pickMethodDict = new AppRewardBackVO.PickMethod();
            pickMethodDict.setValue(pickMethod);
            DictDataRespDTO wayOfReceiving = DictFrameworkUtils.getDictDataFromCache("way_of_receiving", pickMethod);
            if (wayOfReceiving != null) {
                pickMethodDict.setLabelZh(wayOfReceiving.getLabel());
                pickMethodDict.setLabelEn(wayOfReceiving.getLabelEn());
            }
            appRewardBackVO.getPickMethodList().add(pickMethodDict);
        }
    }

    /**
     * 校验礼品是否过期并修改礼品状态
     */
    @Override
    public Boolean validateExpire(RewardDO rewardDO) {
        if (rewardDO.getEndTime() != null) {
            if (rewardDO.getStatus() == RewardStatusEnum.ENABLED.getValue() && rewardDO.getEndTime().toInstant().isBefore(Instant.now())) {
                rewardDO.setStatus(RewardStatusEnum.EXPIRED.getValue());
                RewardDO expireReward = new RewardDO();
                expireReward.setId(rewardDO.getId());
                expireReward.setStatus(RewardStatusEnum.EXPIRED.getValue());
                rewardMapper.updateById(expireReward);
                return true;
            }
        }
        return false;
    }

    /**
     * 获取会员所在城市网点id
     */
    private List<Long> getMemberCityNodes(Long memberId) {
        UserRespDTO user = memberUserApi.getUser(memberId);
        List<WarehouseDO> warehouseList;
        // 如果用户没有填写国家或城市默认展示中国的礼品
        if (user.getCity() == null || user.getCountry() == null) {
            warehouseList = warehouseMapper.selectList(new LambdaQueryWrapperX<WarehouseDO>().eq(WarehouseDO::getGuojia, 130L));
        } else {
            // 根据用户城市获取
            warehouseList = warehouseMapper.selectList(new LambdaQueryWrapperX<WarehouseDO>().eq(WarehouseDO::getShi, user.getCity().longValue()));
            if (CollectionUtil.isEmpty(warehouseList)) {
                // 根据用户国家获取
                warehouseList = warehouseMapper.selectList(new LambdaQueryWrapperX<WarehouseDO>().eq(WarehouseDO::getGuojia, user.getCountry().longValue()));
            }
        }
        // 国家和城市都获取不到
        if (CollectionUtil.isEmpty(warehouseList)) {
            warehouseList = warehouseMapper.selectList(new LambdaQueryWrapperX<WarehouseDO>().eq(WarehouseDO::getGuojia, 130L));
        }
        return warehouseList.stream().map(WarehouseDO::getId).collect(Collectors.toList());
    }
}
